@@ -32,6 +32,10 @@
 #define CMD_MAXLINE 4096
 #define CMD_MAXCMDS 20
 #define CMD_OLDSIZE 4096
+
+#define CONFIG_SIZE 4096
+#define AUTOEXEC_SIZE 4096
+
 extern Bitu call_shellstop;
 class DOS_Shell;
 
@@ -76,13 +80,14 @@ private:
             return strcasecmp(_Left.c_str(), _Right.c_str()) < 0;
         }
     };
-    typedef std::map<std::string, std::string, less_ignore_case<std::string> > cmd_alias_map_t;
+    typedef std::map<std::string, std::string, less_ignore_case<std::string> > cmd_alias_map_t, cmd_assoc_map_t;
     cmd_alias_map_t cmd_alias;
-
+    cmd_assoc_map_t cmd_assoc;
 	uint16_t completion_index;
 	
 private:
 	void ProcessCmdLineEnvVarStitution(char * line);
+	std::string hasAssociation(const char* name);
 	static bool hasExecutableExtension(const char* name);
 
 public:
@@ -136,6 +141,10 @@ public:
      */
 	char * Which(char * name);
 
+    /*! \brief      Command history list
+     */
+	void CMD_HISTORY(char * args);
+
     /*! \brief      Online HELP for the shell
      */
 	void CMD_HELP(char * args);
@@ -305,6 +314,10 @@ public:
     */
 	void CMD_ALIAS(char* args);
 
+    /*! \brief      ALIAS
+    */
+	void CMD_ASSOC(char* args);
+
     /*! \brief      VTEXT
     */
 	void CMD_VTEXT(char *args);
