@@ -19,8 +19,8 @@
 /* Shut up! */
 #define _CRT_NONSTDC_NO_DEPRECATE
 
+#include "bitmapinfoheader.h"
 #include "rawint.h"
-#include "avi.h"
 #include "avi_writer.h"
 #include "avi_rw_iobuf.h"
 #include <unistd.h>
