@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -119,12 +119,12 @@ private:
 			case 0:
 				if ((modrm&7)==5) {
 					// update offset to be RIP relative
-					Bits diff = offset - (Bits)cache.pos - 4 - imm_size;
+					Bits diff = offset - (Bits)cache_rwtox(cache.pos) - 4 - imm_size;
 					if ((int32_t)diff == diff) offset = diff;
 					else { // try 32-bit absolute address
 						if ((int32_t)offset != offset) IllegalOption("opcode::Emit: bad RIP address");
 						// change emitted modrm base from 5 to 4 (use sib)
-						cache.pos[-1] -= 1; 
+						cache_addb(modrm-1,cache.pos-1);
 						cache_addb(0x25); // sib: [none+1*none+simm32]
 					}
 				} else if ((modrm&7)!=4 || (sib&7)!=5)
@@ -275,13 +275,15 @@ static BlockReturn (*gen_runcode)(uint8_t *code) = gen_runcodeInit;
 static BlockReturn gen_runcodeInit(uint8_t *code) {
 	uint8_t* oldpos = cache.pos;
 	cache.pos = &cache_code_link_blocks[128];
-	gen_runcode = (BlockReturn(*)(uint8_t*))cache.pos;
+	gen_runcode = (BlockReturn(*)(uint8_t*))cache_rwtox(cache.pos);
 
 	opcode(5).Emit8Reg(0x50);  // push rbp
 	opcode(15).Emit8Reg(0x50); // push r15
 	opcode(14).Emit8Reg(0x50); // push r14
 	// mov rbp, &cpu_regs
-	if ((uint32_t)(Bitu)&cpu_regs == (Bitu)&cpu_regs) opcode(5).setimm((Bitu)&cpu_regs,4).Emit8Reg(0xB8);
+    /* NTS: For VS2019, mere typecasting won't correctly detect whether the pointer is small enough to fit in 32 bits.
+            We have to mask and compare. If the pointer is above 4GB and the wrong choice is made, dynamic core will crash. --J.C. */
+	if ((((Bitu)&cpu_regs) & (Bitu)0xFFFFFFFFul) == (Bitu)&cpu_regs) opcode(5).setimm((Bitu)&cpu_regs,4).Emit8Reg(0xB8);
 	else opcode(5).set64().setimm((Bitu)&cpu_regs,8).Emit8Reg(0xB8);
 	opcode(13).Emit8Reg(0x50); // push r13
 	opcode(12).Emit8Reg(0x50); // push r12
@@ -475,15 +477,17 @@ static void gen_needcarry(void) {
 	}
 }
 
+/* UNUSED
 static void gen_setzeroflag(void) {
 	if (x64gen.flagsactive) IllegalOption("gen_setzeroflag");
 	opcode(1).setea(4,-1,0,CALLSTACK).setimm(0x40,1).Emit8(0x83); // or dword [rsp+8/40],0x40
-}
+}*/
 
+/* UNUSED
 static void gen_clearzeroflag(void) {
 	if (x64gen.flagsactive) IllegalOption("gen_clearzeroflag");
 	opcode(4).setea(4,-1,0,CALLSTACK).setimm(~0x40,1).Emit8(0x83); // and dword [rsp+8/40],~0x40
-}
+}*/
 
 static bool skip_flags=false;
 
@@ -538,6 +542,26 @@ static void gen_mov_host(void * data,DynReg * dr1,Bitu size,Bitu di1=0) {
 	dr1->flags|=DYNFLG_CHANGED;
 }
 
+static void gen_save_host(void * data,DynReg * dr1,Bitu size,Bitu di1=0) {
+	int idx = FindDynReg(dr1)->index;
+	opcode op;
+	uint8_t tmp;
+	switch (size) {
+	case 1:
+		op.setreg(idx,di1);
+		tmp = 0x88; // mov [], r8
+		break;
+	case 2: op.setword(); // mov [], r16
+	case 4: op.setreg(idx);
+		tmp = 0x89; // mov [], r32
+		break;
+	default:
+		IllegalOption("gen_mov_host");
+	}
+	op.setabsaddr(data).Emit8(tmp);
+	dr1->flags|=DYNFLG_CHANGED;
+}
+
 static void gen_load_arg_reg(int argno,DynReg *dr,const char *s) {
 	GenReg *gen = x64gen.regs[reg_args[argno]];
 	GenReg *src = dr->genreg;
@@ -1026,7 +1050,7 @@ static void gen_call_ptr(void *func=NULL, uint8_t ptr=0) {
 
 	/* Do the actual call to the procedure */
 	if (func!=NULL) {
-		Bits diff = (Bits)func - (Bits)cache.pos - 5;
+		Bits diff = (Bits)func - (Bits)cache_rwtox(cache.pos) - 5;
 		if ((int32_t)diff == diff) {
 			opcode(0).setimm(diff,4).Emit8Reg(0xE8); // call rel32
 			return;
@@ -1116,9 +1140,9 @@ static void gen_call_write(DynReg * dr,uint32_t val,Bitu write_size) {
 	gen_load_arg_reg(0,dr,"rd");
 
 	switch (write_size) {
-		case 1: func = (void*)mem_writeb_checked; break;
-		case 2: func = (void*)mem_writew_checked; break;
-		case 4: func = (void*)mem_writed_checked; break;
+		case 1: func = (void*)(use_dynamic_core_with_paging ? mem_writeb_checked_pagefault : mem_writeb_checked); break;
+		case 2: func = (void*)(use_dynamic_core_with_paging ? mem_writew_checked_pagefault : mem_writew_checked); break;
+		case 4: func = (void*)(use_dynamic_core_with_paging ? mem_writed_checked_pagefault : mem_writed_checked); break;
 		default: IllegalOption("gen_call_write");
 	}
 
@@ -1207,6 +1231,10 @@ static void gen_save_host_direct(void *data,Bitu imm) {
 		opcode(0).set64().setimm(imm,4).setabsaddr(data).Emit8(0xC7); // mov qword[], int32_t
 }
 
+static void gen_test_host_byte(void * data, uint8_t imm) {
+	opcode(0).setimm(imm,1).setabsaddr(data).Emit8(0xF6); // test byte[], uint8_t
+}
+
 static void gen_return(BlockReturn retcode) {
 	gen_protectflags();
 	opcode(1).setea(4,-1,0,CALLSTACK).Emit8(0x8B); // mov ecx, [rsp+8/40]
@@ -1317,7 +1345,7 @@ static void (*gen_dh_fpu_save)(void)  = gen_dh_fpu_saveInit;
 static void gen_dh_fpu_saveInit(void) {
 	uint8_t* oldpos = cache.pos;
 	cache.pos = &cache_code_link_blocks[64];
-	gen_dh_fpu_save = (void(*)(void))cache.pos;
+	gen_dh_fpu_save = (void(*)(void))cache_rwtox(cache.pos);
 
 	Bitu addr = (Bitu)&dyn_dh_fpu;
 	// mov RAX, &dyn_dh_fpu
