@@ -440,13 +440,13 @@ void FPU_ESC3_Normal(Bitu rm) {
 	case 0x04:
 		switch (sub) {
 		case 0x00:				//FNENI
-			if (CPU_ArchitectureType==CPU_ARCHTYPE_8086)
+			if (FPU_ArchitectureType<=FPU_ARCHTYPE_8087)
 				fpu.cw.M = false;
 			else
 				LOG(LOG_FPU,LOG_ERROR)("8087 only fpu code used esc 3: group 4: subfuntion :%d",(int)sub);
 			break;
 		case 0x01:				//FNDIS
-			if (CPU_ArchitectureType==CPU_ARCHTYPE_8086)
+			if (FPU_ArchitectureType<=FPU_ARCHTYPE_8087)
 				fpu.cw.M = true;
 			else
 				LOG(LOG_FPU,LOG_ERROR)("8087 only fpu code used esc 3: group 4: subfuntion :%d",(int)sub);
