@@ -156,8 +156,9 @@ static const char *def_menu_main[] =
     "--",
     "mapper_reset",
     "mapper_reboot",
-    "restartinst",
+    "mapper_pwrbutton", /* APM/ACPI etc */
     "--",
+    "restartinst",
     "restartconf",
     "restartlang",
     "--",
@@ -401,7 +402,7 @@ static const char *def_menu_video_ttf[] =
 {
     "mapper_incsize",
     "mapper_decsize",
-    "ttf_resetcolor",
+    "mapper_resetcolor",
     "--",
     "ttf_showbold",
     "ttf_showital",
@@ -420,8 +421,8 @@ static const char *def_menu_video_ttf[] =
     "ttf_printfont",
 #endif
     "--",
-    "ttf_dbcs_sbcs",
-    "ttf_autoboxdraw",
+    "mapper_dbcssbcs",
+    "mapper_autoboxdraw",
     "ttf_halfwidthkana",
     "ttf_extcharset",
     NULL
@@ -791,6 +792,7 @@ static const char* def_menu_debug[] =
     "save_logas",
     "--",
     "show_console",
+    "disable_logging",
     "wait_on_error",
     "--",
     "debug_logint21",
@@ -3296,4 +3298,4 @@ void DOSBoxMenu::item::drawBackground(DOSBoxMenu &menu) {
                       (int)popupBox.w - (int)DOSBoxMenu::dropshadowX, (int)DOSBoxMenu::dropshadowY);
     }
 }
-#endif
\ No newline at end of file
+#endif
