@@ -14,6 +14,7 @@
  *  You should have received a copy of the GNU General Public License along
  *  with this program; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  VMware mouse protocol support by the DOSBox Staging Team (ported by Wengier)
  */
 
 /* TODO: If biosps2=true and aux=false, also allow an option (default disabled)
@@ -22,6 +23,7 @@
 
 #include <string.h>
 #include <math.h>
+#include <vector>
 
 #include "dosbox.h"
 #include "callback.h"
@@ -45,6 +47,24 @@
 # pragma warning(disable:4244) /* const fmath::local::uint64_t to double possible loss of data */
 #endif
 
+#define VMWARE_PORT         0x5658u        // communication port
+#define VMWARE_PORTHB       0x5659u        // communication port, high bandwidth
+static Bitu PortRead(Bitu port, Bitu iolen);
+
+static constexpr uint32_t TYPE_STANDARD     = 0;
+static constexpr uint32_t TYPE_INTELLIMOUSE = 3;
+
+static constexpr uint32_t RATE_10  = 0; // sample rate 10 Hz
+static constexpr uint32_t RATE_20  = 1;
+static constexpr uint32_t RATE_40  = 2;
+static constexpr uint32_t RATE_60  = 3;
+static constexpr uint32_t RATE_80  = 4;
+static constexpr uint32_t RATE_100 = 5;
+static constexpr uint32_t RATE_200 = 6;
+
+// Unlock sequences taken from https://www.scs.stanford.edu/10wi-cs140/pintos/specs/kbd/scancodes-12.html
+static const std::vector<uint32_t> UNLOCK_SEQ_INTELLIMOUSE = { RATE_200, RATE_100, RATE_80 };
+
 /* ints/bios.cpp */
 void bios_enable_ps2();
 void WriteCharTopView(uint16_t off, int count);
@@ -58,6 +78,7 @@ extern bool MOUSE_IsLocked();
 extern bool usesystemcursor, dbcs_sbcs, showdbcs, del_flag;
 
 bool en_int33=false;
+bool en_vmware=false;
 bool en_bios_ps2mouse=false;
 bool cell_granularity_disable=false;
 bool en_int33_hide_if_polling=false;
@@ -158,6 +179,7 @@ static uint16_t userdefCursorMask[CURSORY];
 
 static struct {
     uint8_t buttons;
+    int16_t wheel;
     uint16_t times_pressed[MOUSE_BUTTONS];
     uint16_t times_released[MOUSE_BUTTONS];
     uint16_t last_released_x[MOUSE_BUTTONS];
@@ -165,6 +187,8 @@ static struct {
     uint16_t last_pressed_x[MOUSE_BUTTONS];
     uint16_t last_pressed_y[MOUSE_BUTTONS];
     pic_tickindex_t hidden_at;
+    uint16_t last_scrolled_x;
+    uint16_t last_scrolled_y;
     uint16_t hidden;
     float add_x,add_y;
     int16_t min_x,max_x,min_y,max_y;
@@ -211,8 +235,30 @@ static struct {
     uint8_t mode;
     int16_t gran_x,gran_y;
     int scrollwheel;
+    uint8_t ps2_type;
+    uint8_t ps2_rate; // sampling rate is not really emulated, but needed for switching between protocols
+    uint8_t ps2_packet_size;
+    uint8_t ps2_unlock_idx;
 } mouse;
 
+double clamp(double d, double min, double max) {
+  const double t = d < min ? min : d;
+  return t > max ? max : t;
+}
+
+inline uint8_t GetWheel8bit() {
+	/* CuteMouse wheel extension allows for -0x80,0x7F range, but let's keep it symmetric */
+	int8_t tmp=clamp(mouse.wheel, static_cast<int16_t>(-0x7F), static_cast<int16_t>(0x7F));
+	mouse.wheel=0;
+	return (tmp >= 0) ? tmp : 0x100 + tmp;
+}
+
+inline uint8_t GetWheel16bit() {
+	int16_t tmp=(mouse.wheel >= 0) ? mouse.wheel : 0x10000 + mouse.wheel;
+	mouse.wheel=0;
+	return tmp;
+}
+
 bool Mouse_SetPS2State(bool use) {
     if (use && (!ps2callbackinit)) {
         useps2callback = false;
@@ -231,6 +277,35 @@ bool Mouse_SetPS2State(bool use) {
     return true;
 }
 
+void Mouse_PS2Reset(void) {
+	mouse.ps2_type       = TYPE_STANDARD;
+	mouse.ps2_rate       = RATE_100;
+	mouse.ps2_unlock_idx = 0;
+}
+
+bool Mouse_PS2SetPacketSize(uint8_t packet_size) {
+	if ((packet_size==0x03) ||
+		(packet_size==0x04 && mouse.ps2_type==TYPE_INTELLIMOUSE)) {
+		mouse.ps2_packet_size = packet_size;
+	    return true;
+	}
+	return false;
+}
+
+void Mouse_PS2SetSamplingRate(uint8_t rate) {
+	mouse.ps2_rate = rate;
+	if (UNLOCK_SEQ_INTELLIMOUSE[mouse.ps2_unlock_idx]!=rate)
+		mouse.ps2_unlock_idx = 0;
+	else if (UNLOCK_SEQ_INTELLIMOUSE.size()==++mouse.ps2_unlock_idx) {
+			mouse.ps2_type=TYPE_INTELLIMOUSE;
+			mouse.ps2_unlock_idx=0;
+	}
+}
+
+uint8_t Mouse_PS2GetType(void) {
+	return mouse.ps2_type;
+}
+
 void Mouse_ChangePS2Callback(uint16_t pseg, uint16_t pofs) {
     if ((pseg==0) && (pofs==0)) {
         ps2callbackinit = false;
@@ -270,10 +345,18 @@ void DoPS2Callback(uint16_t data, int16_t mouseX, int16_t mouseY) {
             CPU_Push16(reg_bp);CPU_Push16(reg_si);CPU_Push16(reg_di);
             CPU_Push16(SegValue(ds)); CPU_Push16(SegValue(es));
         }
-        CPU_Push16((uint16_t)mdat); 
-        CPU_Push16((uint16_t)(xdiff % 256)); 
-        CPU_Push16((uint16_t)(ydiff % 256)); 
-        CPU_Push16((uint16_t)0); 
+        switch (mouse.ps2_packet_size) {
+            case 0x04: // IntelliMouse protocol
+                CPU_Push16((uint16_t)(mdat + (xdiff % 256) * 256));
+                CPU_Push16((uint16_t)(ydiff % 256));
+                CPU_Push16((uint16_t)GetWheel8bit());
+                CPU_Push16((uint16_t)0);
+            default:   // Standard protocol
+                CPU_Push16((uint16_t)mdat);
+                CPU_Push16((uint16_t)(xdiff % 256));
+                CPU_Push16((uint16_t)(ydiff % 256));
+                CPU_Push16((uint16_t)0);
+        }
         CPU_Push16(RealSeg(ps2_callback));
         CPU_Push16(RealOff(ps2_callback));
         SegSet16(cs, ps2cbseg);
@@ -302,7 +385,8 @@ Bitu PS2_Handler(void) {
 #define MOUSE_RIGHT_RELEASED 16
 #define MOUSE_MIDDLE_PRESSED 32
 #define MOUSE_MIDDLE_RELEASED 64
-#define MOUSE_DUMMY 128
+#define MOUSE_WHEEL_MOVED 128
+#define MOUSE_DUMMY 256
 #define MOUSE_DELAY 5.0
 
 void MOUSE_Limit_Events(Bitu /*val*/) {
@@ -329,7 +413,7 @@ INLINE void Mouse_AddEvent(uint8_t type) {
     if (mouse.events<QUEUE_SIZE) {
         if (mouse.events>0) {
             /* Skip duplicate events */
-            if (type==MOUSE_HAS_MOVED) return;
+            if (type==MOUSE_HAS_MOVED || type==MOUSE_WHEEL_MOVED) return;
             /* Always put the newest element in the front as that the events are 
              * handled backwards (prevents doubleclicks while moving)
              */
@@ -1078,6 +1162,13 @@ void Mouse_ButtonReleased(uint8_t button) {
     on_mouse_event_for_serial(0,0,mouse.buttons);
 }
 
+void Mouse_WheelMoved(int32_t scroll) {
+	mouse.wheel = clamp(scroll + mouse.wheel, -0x7FFF, 0x7FFF); /* limit is -0x8000,0x7FFF, but let's keep it symmetric */
+	Mouse_AddEvent(MOUSE_WHEEL_MOVED);
+	mouse.last_scrolled_x=POS_X;
+	mouse.last_scrolled_y=POS_Y;
+}
+
 static void Mouse_SetMickeyPixelRate(int16_t px, int16_t py){
     if ((px!=0) && (py!=0)) {
         mouse.mickeysPerPixel_x  = (float)px/X_MICKEY;
@@ -1267,6 +1358,10 @@ static void Mouse_Reset(void) {
 
     mouse.buttons = 0;
 
+    mouse.wheel           = 0; /* CuteMouse wheel extension */
+    mouse.last_scrolled_x = 0;
+    mouse.last_scrolled_y = 0;
+
     for (uint16_t but=0; but<MOUSE_BUTTONS; but++) {
         mouse.times_pressed[but] = 0;
         mouse.times_released[but] = 0;
@@ -1323,9 +1418,10 @@ static Bitu INT33_Handler(void) {
         }
         break;
     case 0x03:  /* MS MOUSE v1.0+ - RETURN POSITION AND BUTTON STATUS */
-        reg_bx = mouse.buttons;
-        reg_cx = (uint16_t)POS_X;
-        reg_dx = (uint16_t)POS_Y;
+        reg_bl=mouse.buttons;
+        reg_bh=GetWheel8bit(); /* CuteMouse wheel extension */
+        reg_cx=POS_X;
+        reg_dx=POS_Y;
         mouse.first_range_setx = false;
         mouse.first_range_sety = false;
         if (en_int33_hide_if_polling) int33_last_poll = PIC_FullIndex();
@@ -1348,12 +1444,19 @@ static Bitu INT33_Handler(void) {
     case 0x05:  /* MS MOUSE v1.0+ - RETURN BUTTON PRESS DATA */
         {
             uint16_t but = reg_bx;
-            reg_ax = mouse.buttons;
-            if (but >= MOUSE_BUTTONS) but = MOUSE_BUTTONS - 1;
-            reg_cx = mouse.last_pressed_x[but];
-            reg_dx = mouse.last_pressed_y[but];
-            reg_bx = mouse.times_pressed[but];
-            mouse.times_pressed[but] = 0;
+            if (but==0xFFFF){
+			    /* CuteMouse wheel extension */
+			    reg_bx=GetWheel16bit();
+			    reg_cx=mouse.last_scrolled_x;
+			    reg_dx=mouse.last_scrolled_y;
+			} else {
+				reg_ax=mouse.buttons;
+				if (but>=MOUSE_BUTTONS) but = MOUSE_BUTTONS - 1;
+				reg_cx=mouse.last_pressed_x[but];
+				reg_dx=mouse.last_pressed_y[but];
+				reg_bx=mouse.times_pressed[but];
+				mouse.times_pressed[but]=0;
+			}
             if (en_int33_hide_if_polling) int33_last_poll = PIC_FullIndex();
         }
         Mouse_Used();
@@ -1361,12 +1464,20 @@ static Bitu INT33_Handler(void) {
     case 0x06:  /* MS MOUSE v1.0+ - RETURN BUTTON RELEASE DATA */
         {
             uint16_t but = reg_bx;
-            reg_ax = mouse.buttons;
-            if (but >= MOUSE_BUTTONS) but = MOUSE_BUTTONS - 1;
-            reg_cx = mouse.last_released_x[but];
-            reg_dx = mouse.last_released_y[but];
-            reg_bx = mouse.times_released[but];
-            mouse.times_released[but] = 0;
+            if (but==0xFFFF){
+			    /* CuteMouse wheel extension */
+			    reg_bx=(mouse.wheel >= 0) ? mouse.wheel : (0x10000 + mouse.wheel);
+			    reg_cx=mouse.last_scrolled_x;
+			    reg_dx=mouse.last_scrolled_y;
+			    mouse.wheel=0;
+			} else {
+				reg_ax=mouse.buttons;
+				if (but>=MOUSE_BUTTONS) but = MOUSE_BUTTONS - 1;
+				reg_cx=mouse.last_released_x[but];
+				reg_dx=mouse.last_released_y[but];
+				reg_bx=mouse.times_released[but];
+				mouse.times_released[but]=0;
+			}
             if (en_int33_hide_if_polling) int33_last_poll = PIC_FullIndex();
         }
         Mouse_Used();
@@ -1542,8 +1653,13 @@ static Bitu INT33_Handler(void) {
         DrawCursor();
         break;
     case 0x11:  /* Genius Mouse 9.06 - GET NUMBER OF BUTTONS */
-        reg_ax = 0xffff;
-        reg_bx = MOUSE_BUTTONS;
+        reg_ax = 0x574D; /* Identifier for detection purposes */
+		reg_bx = 0;      /* Reserved capabilities flags */
+		reg_cx = 1;      /* Wheel is supported */
+		/* Previous implementation provided Genius mouse-specific function to get
+		   number of buttons (https://sourceforge.net/p/dosbox/patches/32/), it was
+		   returning 0xffff in reg_ax and number of buttons in reg_bx; I suppose
+		   the CuteMouse extensions are more useful */
         break;
     case 0x12:  /* MS MOUSE - SET LARGE GRAPHICS CURSOR BLOCK */
         LOG(LOG_MOUSE, LOG_ERROR)("Set large graphics cursor block not implemented");
@@ -1840,7 +1956,8 @@ static Bitu INT74_Handler(void) {
         /* Check for an active Interrupt Handler that will get called */
         if (AllowINT33RMAccess() && (mouse.sub_mask & mouse.event_queue[mouse.events].type)) {
             reg_ax=mouse.event_queue[mouse.events].type;
-            reg_bx=mouse.event_queue[mouse.events].buttons;
+            reg_bl=mouse.event_queue[mouse.events].buttons;
+            reg_bh=GetWheel8bit(); /* CuteMouse wheel extension */
             reg_cx=(uint16_t)POS_X;
             reg_dx=(uint16_t)POS_Y;
             reg_si=(uint16_t)static_cast<int16_t>(mouse.mickey_x);
@@ -1995,6 +2112,8 @@ void MOUSE_Startup(Section *sec) {
 
     en_int33_pc98_show_graphics=pc98_section->Get_bool("pc-98 show graphics layer on initialize");
 
+    en_vmware=section->Get_bool("vmware");
+
     en_int33=section->Get_bool("int33");
     if (!en_int33) {
         Mouse_Reset();
@@ -2048,6 +2167,11 @@ void MOUSE_Startup(Section *sec) {
     Mouse_ResetHardware();
     Mouse_Reset();
     Mouse_SetSensitivity(50,50,50);
+
+    if (en_vmware) {
+	    LOG(LOG_MISC,LOG_DEBUG)("Enabling VMware integration for mouse interface");
+	    IO_RegisterReadHandler(VMWARE_PORT, &PortRead, IO_MD);
+    }
 }
 
 void MOUSE_Init() {
@@ -2080,6 +2204,221 @@ bool MOUSE_HasInterruptSub()
     return (mouse.sub_mask != 0);
 }
 
+// Basic VMware tools support, based on documentation from https://wiki.osdev.org/VMware_tools
+// Mouse support tested using unofficial Windows 3.1 driver from https://github.com/NattyNarwhal/vmwmouse
+
+static constexpr uint32_t VMWARE_MAGIC           = 0x564D5868u;    // magic number for all VMware calls
+
+static constexpr uint16_t CMD_GETVERSION         = 10u;
+static constexpr uint16_t CMD_ABSPOINTER_DATA    = 39u;
+static constexpr uint16_t CMD_ABSPOINTER_STATUS  = 40u;
+static constexpr uint16_t CMD_ABSPOINTER_COMMAND = 41u;
+
+static constexpr uint32_t ABSPOINTER_ENABLE      = 0x45414552u;
+static constexpr uint32_t ABSPOINTER_RELATIVE    = 0xF5u;
+static constexpr uint32_t ABSPOINTER_ABSOLUTE    = 0x53424152u;
+
+static constexpr uint8_t BUTTON_LEFT            = 0x20u;
+static constexpr uint8_t BUTTON_RIGHT           = 0x10u;
+static constexpr uint8_t BUTTON_MIDDLE          = 0x08u;
+
+volatile bool vmware_mouse     = false;  // if true, VMware compatible driver has taken over the mouse
+
+static uint8_t mouse_buttons    = 0;      // state of mouse buttons, in VMware format
+static uint16_t mouse_x          = 0x8000; // mouse position X, in VMware format (scaled from 0 to 0xFFFF)
+static uint16_t mouse_y          = 0x8000; // ditto
+static int8_t mouse_wheel      = 0;
+static bool mouse_updated    = false;
+
+static int16_t mouse_diff_x     = 0;      // difference between host and guest mouse x coordinate (in host pixels)
+static int16_t mouse_diff_y     = 0;      // ditto
+
+static bool video_fullscreen = false;
+static uint16_t video_res_x      = 1;      // resolution to which guest image is scaled, excluding black borders
+static uint16_t video_res_y      = 1;
+static uint16_t video_clip_x     = 0;      // clipping value - size of black border (one side)
+static uint16_t video_clip_y     = 0;
+
+
+class Section;
+
+// Commands (requests) to the VMware hypervisor
+
+static inline void CmdGetVersion() {
+        reg_eax = 0; // FIXME: should we respond with something resembling VMware?
+        reg_ebx = VMWARE_MAGIC;
+}
+
+static inline void CmdAbsPointerData() {
+        reg_eax = mouse_buttons;
+        reg_ebx = mouse_x;
+        reg_ecx = mouse_y;
+        reg_edx = (mouse_wheel >= 0) ? mouse_wheel : 256 + mouse_wheel;
+
+        mouse_wheel = 0;
+}
+
+static inline void CmdAbsPointerStatus() {
+        reg_eax = mouse_updated ? 4 : 0;
+        mouse_updated = false;
+}
+
+static inline void CmdAbsPointerCommand() {
+
+        switch (reg_ebx) {
+        case ABSPOINTER_ENABLE:
+                // can be safely ignored
+                break;
+        case ABSPOINTER_RELATIVE:
+                vmware_mouse = false;
+                if (!MOUSE_IsLocked()) SDL_ShowCursor(SDL_ENABLE);
+                break;
+        case ABSPOINTER_ABSOLUTE:
+                vmware_mouse = true;
+                if (!MOUSE_IsLocked()) SDL_ShowCursor(SDL_DISABLE);
+                break;
+        default:
+                LOG_MSG("VMWARE: unknown mouse subcommand 0x%08x", reg_ebx);
+                break;
+        }
+}
+
+// IO port handling
+
+static Bitu PortRead(Bitu port, Bitu iolen) {
+
+        if (reg_eax != VMWARE_MAGIC)
+                return 0;
+
+        // LOG_MSG("VMWARE: called with EBX 0x%08x, ECX 0x%08x", reg_ebx, reg_ecx);
+
+        switch (reg_cx) {
+        case CMD_GETVERSION:
+                CmdGetVersion();
+                break;
+        case CMD_ABSPOINTER_DATA:
+                CmdAbsPointerData();
+                break;
+        case CMD_ABSPOINTER_STATUS:
+                CmdAbsPointerStatus();
+                break;
+        case CMD_ABSPOINTER_COMMAND:
+                CmdAbsPointerCommand();
+                break;
+        default:
+                LOG_MSG("VMWARE: unknown command 0x%08x", reg_ecx);
+                break;
+        }
+
+        return reg_ax;
+}
+
+// Notifications from external subsystems
+
+void VMWARE_MouseButtonPressed(uint8_t button) {
+
+        switch (button) {
+        case 0:
+                mouse_buttons |= BUTTON_LEFT;
+                mouse_updated = true;
+                break;
+        case 1:
+                mouse_buttons |= BUTTON_RIGHT;
+                mouse_updated = true;
+                break;
+        case 2:
+                mouse_buttons |= BUTTON_MIDDLE;
+                mouse_updated = true;
+                break;
+        default:
+                break;
+        }
+}
+
+void VMWARE_MouseButtonReleased(uint8_t button) {
+
+        switch (button) {
+        case 0:
+                mouse_buttons &= ~BUTTON_LEFT;
+                mouse_updated = true;
+                break;
+        case 1:
+                mouse_buttons &= ~BUTTON_RIGHT;
+                mouse_updated = true;
+                break;
+        case 2:
+                mouse_buttons &= ~BUTTON_MIDDLE;
+                mouse_updated = true;
+                break;
+        default:
+                break;
+        }
+}
+
+void VMWARE_MousePosition(uint16_t pos_x, uint16_t pos_y) {
+
+        float tmp_x;
+        float tmp_y;
+
+        if (video_fullscreen)
+        {
+                // We have to maintain the diffs (offsets) between host and guest
+                // mouse positions; otherwise in case of clipped picture (like
+                // 4:3 screen displayed on 16:9 fullscreen mode) we could have
+                // an effect of 'sticky' borders if the user moves mouse outside
+                // of the guest display area
+
+                if (pos_x + mouse_diff_x < video_clip_x)
+                        mouse_diff_x = video_clip_x - pos_x;
+                else if (pos_x + mouse_diff_x >= video_res_x + video_clip_x)
+                        mouse_diff_x = video_res_x + video_clip_x - pos_x - 1;
+
+                if (pos_y + mouse_diff_y < video_clip_y)
+                        mouse_diff_y = video_clip_y - pos_y;
+                else if (pos_y + mouse_diff_y >= video_res_y + video_clip_y)
+                        mouse_diff_y = video_res_y + video_clip_y - pos_y - 1;
+
+                tmp_x = pos_x + mouse_diff_x - video_clip_x;
+                tmp_y = pos_y + mouse_diff_y - video_clip_y;
+        }
+        else
+        {
+                tmp_x = pos_x > video_clip_x ? pos_x - video_clip_x : 0;
+                tmp_y = pos_y > video_clip_y ? pos_y - video_clip_y : 0;
+        }
+
+        mouse_x = (std::min)(0xFFFFu, static_cast<uint32_t>(tmp_x * 0xFFFF / (video_res_x - 1) + 0.499));
+        mouse_y = (std::min)(0xFFFFu, static_cast<uint32_t>(tmp_y * 0xFFFF / (video_res_y - 1) + 0.499));
+
+        mouse_updated = true;
+}
+
+void VMWARE_MouseWheel(int32_t scroll) {
+        if (scroll >= 255 || scroll + mouse_wheel >= 127)
+                mouse_wheel = 127;
+        else if (scroll <= -255 || scroll + mouse_wheel <= -127)
+                mouse_wheel = -127;
+        else
+                mouse_wheel += scroll;
+
+        mouse_updated = true;
+}
+
+void VMWARE_ScreenParams(uint16_t clip_x, uint16_t clip_y, uint16_t res_x, uint16_t res_y, bool fullscreen) {
+        video_clip_x     = clip_x;
+        video_clip_y     = clip_y;
+        video_res_x      = res_x;
+        video_res_y      = res_y;
+        video_fullscreen = fullscreen;
+
+        // Unfortunately, with seamless driver changing the window size can cause
+        // mouse movement as a side-effect, this is not fun for games. Let's try
+        // to at least minimize the effect.
+
+        mouse_diff_x = clamp(static_cast<int32_t>(mouse_diff_x), -video_clip_x, static_cast<int32_t>(video_clip_x));
+        mouse_diff_y = clamp(static_cast<int32_t>(mouse_diff_y), -video_clip_y, static_cast<int32_t>(video_clip_y));
+}
+
 //save state support
 void *MOUSE_Limit_Events_PIC_Event = (void*)((uintptr_t)MOUSE_Limit_Events);
 
