@@ -25,6 +25,7 @@
 #include "logging.h"
 #include "support.h"
 #include "setup.h"
+#include "render.h"
 #include "control.h"
 #include "menu.h"
 #include "jfont.h"
@@ -33,7 +34,7 @@
 #include <string>
 using namespace std;
 
-extern bool dos_kernel_disabled, force_conversion, showdbcs;
+extern bool dos_kernel_disabled, force_conversion, showdbcs, dbcs_sbcs;
 int msgcodepage = 0, FileDirExistUTF8(std::string &localname, const char *name);
 bool morelen = false, inmsg = false, loadlang = false, systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
 bool isSupportedCP(int newCP), CodePageHostToGuestUTF8(char *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/), CodePageGuestToHostUTF8(char *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/);
@@ -174,7 +175,7 @@ void AddMessages() {
     MSG_Add("CONFIG_SAVEALL", "Save all (including advanced) config options to the configuration file");
     MSG_Add("LANG_FILENAME", "Enter filename for language file:");
     MSG_Add("LANG_LANGNAME", "Language name (optional):");
-    MSG_Add("INTRO_MESSAGE", "Welcome to DOSBox-X, a free and complete DOS emulation package.\nDOSBox-X creates a DOS shell which looks like the plain DOS.\nYou can also run Windows 3.x and 95/98 inside the DOS machine.");
+    MSG_Add("INTRO_MESSAGE", "Welcome to DOSBox-X, a free and complete DOS emulation package.\nDOSBox-X creates a DOS shell which looks like the plain DOS.\nYou can also run Windows 3.x and 9x/Me inside the DOS machine.");
     MSG_Add("DRIVE","Drive");
     MSG_Add("TYPE","Type");
     MSG_Add("LABEL","Label");
@@ -421,5 +422,9 @@ void MSG_Init() {
         }
 	}
     std::string showdbcsstr = static_cast<Section_prop *>(control->GetSection("dosv"))->Get_string("showdbcsnodosv");
+#if defined(USE_TTF)
+    showdbcs = showdbcsstr=="true"||showdbcsstr=="1"||(showdbcsstr=="auto" && (loadlang || dbcs_sbcs));
+#else
     showdbcs = showdbcsstr=="true"||showdbcsstr=="1"||(showdbcsstr=="auto" && loadlang);
+#endif
 }
