@@ -25,6 +25,7 @@ bool informd3d = false;
 
 #if (HAVE_D3D9_H) && defined(WIN32)
 
+int FileDirExistCP(const char *name);
 extern Bitu currentWindowWidth, currentWindowHeight;
 
 #include "direct3d.h"
@@ -57,7 +58,7 @@ std::string shader_translate_directory(const std::string& path) {
     if (path == "none")
         return path;
 
-    /* DOSBox fork compatability: if only the name of a file is given, assume it
+    /* DOSBox fork compatibility: if only the name of a file is given, assume it
        exists in the shaders\ directory.
 
        DOSBox-X's variation is to NOT prefix shaders\ to it if it looks like a
@@ -66,6 +67,12 @@ std::string shader_translate_directory(const std::string& path) {
         return path;
     if (path.length() >= 1 && path.find('\\') != std::string::npos) /* perhaps a path with "\" */
         return path;
+    if (FileDirExistCP(path.c_str())==1)
+        return path;
+    if (FileDirExistCP((path+".fx").c_str())==1)
+        return path+".fx";
+    if (FileDirExistCP(("shaders\\"+path+".fx").c_str())==1)
+        return "shaders\\"+path+".fx";
 
     return std::string("shaders\\") + path;
 }
@@ -998,7 +1005,7 @@ HRESULT CDirect3D::LoadPixelShader(const char * shader, double scalex, double sc
 	// Compare optimal scaling factor
 	bool dblgfx=((scalex < scaley ? scalex : scaley) >= psEffect->getScale());
 
-    std::string message = "This pixel shader may be unneeded or have undesired effect:\n\n"+std::string(shader)+"\n\nDo you want to load the pixel shader anyway?";
+	std::string message = "This pixel shader may be unneeded or have undesired effect:\n\n"+std::string(shader)+"\n\nDo you want to load the pixel shader anyway?\n\n(You may append 'forced' to the pixelshader setting to force load the pixel shader without this message)";
 	if(dblgfx || forced || systemmessagebox("Direct3D shader", message.c_str(), "yesno","question", 1)) {
 	    message = "Loaded pixel shader - "+std::string(shader);
 	    if (informd3d) systemmessagebox("Direct3D shader", message.c_str(), "ok","info", 1);
@@ -1007,6 +1014,7 @@ HRESULT CDirect3D::LoadPixelShader(const char * shader, double scalex, double sc
 	    psActive = true;
 	    return S_OK;
 	} else {
+	    if (!dblgfx && !forced) SetVal("render", "pixelshader", "none");
 	    LOG_MSG("D3D:Pixel shader not needed");
 	    psActive = false;
 	    return E_FAIL;
