@@ -113,7 +113,7 @@ static int prev_sline = -1;
 static int charSet = 0;
 static alt_rgb *rgbColors = (alt_rgb*)render.pal.rgb;
 static bool blinkstate = false;
-bool colorChanged = false, justChanged = false, firstsize = true;
+bool colorChanged = false, justChanged = false, staycolors = false, firstsize = true;
 
 int menuwidth_atleast(int width), FileDirExistCP(const char *name), FileDirExistUTF8(std::string &localname, const char *name);
 void AdjustIMEFontSize(void), initcodepagefont(void), MSG_Init(void), DOSBox_SetSysMenu(void), GetMaxWidthHeight(unsigned int *pmaxWidth, unsigned int *pmaxHeight), resetFontSize(void), RENDER_CallBack( GFX_CallBackFunctions_t function );
@@ -253,7 +253,8 @@ bool setColors(const char *colorArray, int n) {
             altBGR1[i].green=rgbColors[i].green;
             altBGR1[i].blue=rgbColors[i].blue;
         }
-    const char* nextRGB = colorArray;
+    staycolors = strlen(colorArray) && *colorArray == '+';
+    const char* nextRGB = colorArray + (staycolors?1:0);
 	uint8_t * altPtr = (uint8_t *)altBGR1;
 	int rgbVal[3] = {-1,-1,-1};
 	for (int colNo = 0; colNo < (n>-1?1:16); colNo++) {
@@ -829,12 +830,13 @@ resize:
     if (fontSize>=MIN_PTSIZE && 100*ttf.cols*ttf.width/maxWidth*ttf.lins*ttf.height/maxHeight > 100)
         E_Exit("Cannot accommodate a window for %dx%d", ttf.lins, ttf.cols);
     if (ttf.SDL_font && ttf.width) {
-        int widthb, widthm, widthx, width1, width9;
+        int widthb, widthm, widthx, width0, width1, width9;
         widthb = widthm = widthx = width1 = width9 = 0;
         TTF_GlyphMetrics(ttf.SDL_font, 'B', NULL, NULL, NULL, NULL, &widthb);
         TTF_GlyphMetrics(ttf.SDL_font, 'M', NULL, NULL, NULL, NULL, &widthm);
         TTF_GlyphMetrics(ttf.SDL_font, 'X', NULL, NULL, NULL, NULL, &widthx);
-        if (ttf.width != widthb || ttf.width != widthm || ttf.width != widthx) LOG_MSG("TTF: The loaded font is not monospaced.");
+        TTF_GlyphMetrics(ttf.SDL_font, '0', NULL, NULL, NULL, NULL, &width0);
+        if (abs(ttf.width-widthb)>1 || abs(ttf.width-widthm)>1 || abs(ttf.width-widthx)>1 || abs(ttf.width-width0)>1) LOG_MSG("TTF: The loaded font is not monospaced.");
         int cp=dos.loaded_codepage;
         if (!cp) InitCodePage();
         if ((IS_PC98_ARCH || isDBCSCP()) && dbcs_sbcs) {
@@ -842,7 +844,7 @@ resize:
             TTF_GlyphMetrics(ttf.SDL_font, 0x4E00, NULL, NULL, NULL, NULL, &width1);
             TTF_GlyphMetrics(ttf.SDL_font, 0x4E5D, NULL, NULL, NULL, NULL, &width9);
             if (width1 <= ttf.width || width9 <= ttf.width) LOG_MSG("TTF: The loaded font may not support DBCS characters.");
-            else if ((ttf.width*2 != width1 || ttf.width*2 != width9) && ttf.width == widthb && ttf.width == widthm && ttf.width == widthx) LOG_MSG("TTF: The loaded font is not monospaced.");
+            else if ((ttf.width*2 != width1 || ttf.width*2 != width9) && ttf.width == widthb && ttf.width == widthm && ttf.width == widthx && ttf.width == width0) LOG_MSG("TTF: The loaded font is not monospaced.");
         }
         dos.loaded_codepage = cp;
     }
@@ -950,6 +952,7 @@ void processWP(uint8_t *pcolorBG, uint8_t *pcolorFG) {
     *pcolorFG = colorFG;
 }
 
+bool hasfocus = true, lastfocus = true;
 void GFX_EndTextLines(bool force) {
     if (!force&&!IS_PC98_ARCH&&((!CurMode||CurMode->type!=M_TEXT))) return;
 #if DOSBOXMENU_TYPE == DOSBOXMENU_SDLDRAW
@@ -1006,13 +1009,22 @@ void GFX_EndTextLines(bool force) {
             curAC[ttf.cursor].chr ^= 0xf0f0;	// force redraw (differs)
         }
 
+	lastfocus = hasfocus;
+	hasfocus =
+#if defined(C_SDL2)
+	SDL_GetWindowFlags(sdl.window) & SDL_WINDOW_INPUT_FOCUS
+#else
+	SDL_GetAppState () & SDL_APPINPUTFOCUS
+#endif
+	? true : false;
+	bool focuschanged = lastfocus != hasfocus, noframe = !menu.toggle || ttf.fullScrn;
 	ttf_textClip.h = ttf.height;
 	ttf_textClip.y = 0;
 	for (unsigned int y = 0; y < ttf.lins; y++) {
 		bool draw = false;
 		ttf_textRect.y = ttf.offY+y*ttf.height;
 		for (unsigned int x = 0; x < ttf.cols; x++) {
-			if ((newAC[x] != curAC[x] || newAC[x].selected != curAC[x].selected || (colorChanged && (justChanged || draw)) || force) && !(newAC[x].skipped)) {
+			if ((newAC[x] != curAC[x] || newAC[x].selected != curAC[x].selected || (colorChanged && (justChanged || draw)) || force) && !(newAC[x].skipped) || (!y && focuschanged && noframe)) {
 				draw = true;
 				xmin = min((int)x, xmin);
 				ymin = min((int)y, ymin);
@@ -1028,14 +1040,14 @@ void GFX_EndTextLines(bool force) {
                     colorBG = colorFG;
                     colorFG = color;
                 }
-                bool colornul = IS_VGA_ARCH && (altBGR1[colorBG&15].red > 4 || altBGR1[colorBG&15].green > 4 || altBGR1[colorBG&15].blue > 4 || altBGR1[colorFG&15].red > 4 || altBGR1[colorFG&15].green > 4 || altBGR1[colorFG&15].blue > 4) && rgbColors[colorBG].red < 5 && rgbColors[colorBG].green < 5 && rgbColors[colorBG].blue < 5 && rgbColors[colorFG].red < 5 && rgbColors[colorFG].green <5 && rgbColors[colorFG].blue < 5;
+                bool colornul = staycolors || (IS_VGA_ARCH && (altBGR1[colorBG&15].red > 4 || altBGR1[colorBG&15].green > 4 || altBGR1[colorBG&15].blue > 4 || altBGR1[colorFG&15].red > 4 || altBGR1[colorFG&15].green > 4 || altBGR1[colorFG&15].blue > 4) && rgbColors[colorBG].red < 5 && rgbColors[colorBG].green < 5 && rgbColors[colorBG].blue < 5 && rgbColors[colorFG].red < 5 && rgbColors[colorFG].green <5 && rgbColors[colorFG].blue < 5);
 				ttf_textRect.x = ttf.offX+(rtl?(ttf.cols-x-1):x)*ttf.width;
-				ttf_bgColor.r = colornul||(colorChanged&&!IS_VGA_ARCH)?altBGR1[colorBG&15].red:rgbColors[colorBG].red;
-				ttf_bgColor.g = colornul||(colorChanged&&!IS_VGA_ARCH)?altBGR1[colorBG&15].green:rgbColors[colorBG].green;
-				ttf_bgColor.b = colornul||(colorChanged&&!IS_VGA_ARCH)?altBGR1[colorBG&15].blue:rgbColors[colorBG].blue;
-				ttf_fgColor.r = colornul||(colorChanged&&!IS_VGA_ARCH)?altBGR1[colorFG&15].red:rgbColors[colorFG].red;
-				ttf_fgColor.g = colornul||(colorChanged&&!IS_VGA_ARCH)?altBGR1[colorFG&15].green:rgbColors[colorFG].green;
-				ttf_fgColor.b = colornul||(colorChanged&&!IS_VGA_ARCH)?altBGR1[colorFG&15].blue:rgbColors[colorFG].blue;
+				ttf_bgColor.r = !y&&!hasfocus&&noframe?altBGR0[colorBG&15].red:(colornul||(colorChanged&&!IS_VGA_ARCH)?altBGR1[colorBG&15].red:rgbColors[colorBG].red);
+				ttf_bgColor.g = !y&&!hasfocus&&noframe?altBGR0[colorBG&15].green:(colornul||(colorChanged&&!IS_VGA_ARCH)?altBGR1[colorBG&15].green:rgbColors[colorBG].green);
+				ttf_bgColor.b = !y&&!hasfocus&&noframe?altBGR0[colorBG&15].blue:(colornul||(colorChanged&&!IS_VGA_ARCH)?altBGR1[colorBG&15].blue:rgbColors[colorBG].blue);
+				ttf_fgColor.r = !y&&!hasfocus&&noframe?altBGR0[colorFG&15].red:(colornul||(colorChanged&&!IS_VGA_ARCH)?altBGR1[colorFG&15].red:rgbColors[colorFG].red);
+				ttf_fgColor.g = !y&&!hasfocus&&noframe?altBGR0[colorFG&15].green:(colornul||(colorChanged&&!IS_VGA_ARCH)?altBGR1[colorFG&15].green:rgbColors[colorFG].green);
+				ttf_fgColor.b = !y&&!hasfocus&&noframe?altBGR0[colorFG&15].blue:(colornul||(colorChanged&&!IS_VGA_ARCH)?altBGR1[colorFG&15].blue:rgbColors[colorFG].blue);
 
                 if (newAC[x].unicode) {
                     dw = newAC[x].doublewide;
@@ -1247,6 +1259,49 @@ void TTF_DecreaseSize(bool pressed) {
     return;
 }
 
+void DBCSSBCS_mapper_shortcut(bool pressed) {
+    if (!pressed) return;
+    if (!isDBCSCP()) {
+        systemmessagebox("Warning", "This function is only available for the Chinese/Japanese/Korean code pages.", "ok","warning", 1);
+        return;
+    }
+    dbcs_sbcs=!dbcs_sbcs;
+    SetVal("ttf", "autodbcs", dbcs_sbcs?"true":"false");
+    mainMenu.get_item("mapper_dbcssbcs").check(dbcs_sbcs).refresh_item(mainMenu);
+    if (ttf.inUse) resetFontSize();
+}
+
+void AutoBoxDraw_mapper_shortcut(bool pressed) {
+    if (!pressed) return;
+    if (!isDBCSCP()) {
+        systemmessagebox("Warning", "This function is only available for the Chinese/Japanese/Korean code pages.", "ok","warning", 1);
+        return;
+    }
+    autoboxdraw=!autoboxdraw;
+    SetVal("ttf", "autoboxdraw", autoboxdraw?"true":"false");
+    mainMenu.get_item("mapper_autoboxdraw").check(autoboxdraw).refresh_item(mainMenu);
+    if (ttf.inUse) resetFontSize();
+}
+
+bool setVGAColor(const char *colorArray, int i);
+void ttf_reset_colors() {
+    if (ttf.inUse) {
+        SetVal("ttf", "colors", "");
+        setColors("#000000 #0000aa #00aa00 #00aaaa #aa0000 #aa00aa #aa5500 #aaaaaa #555555 #5555ff #55ff55 #55ffff #ff5555 #ff55ff #ffff55 #ffffff",-1);
+    } else {
+        char value[128];
+        for (int i=0; i<16; i++) {
+            strcpy(value,i==0?"#000000":i==1?"#0000aa":i==2?"#00aa00":i==3?"#00aaaa":i==4?"#aa0000":i==5?"#aa00aa":i==6?"#aa5500":i==7?"#aaaaaa":i==8?"#555555":i==9?"#5555ff":i==10?"#55ff55":i==11?"#55ffff":i==12?"#ff5555":i==13?"#ff55ff":i==14?"#ffff55":"#ffffff");
+            setVGAColor(value, i);
+        }
+    }
+}
+
+void ResetColors_mapper_shortcut(bool pressed) {
+    if (!pressed||(!ttf.inUse&&!IS_VGA_ARCH)) return;
+    ttf_reset_colors();
+}
+
 void ttf_reset() {
     OUTPUT_TTF_Select(2);
     resetFontSize();
